@@ -11,6 +11,9 @@
 #include <gdk/gdkx.h>
 #include "xapp-gtk-window.h"
 
+#define DEBUG_FLAG XAPP_DEBUG_WINDOW
+#include "xapp-debug.h"
+
 #define ICON_NAME_HINT "_NET_WM_XAPP_ICON_NAME"
 #define PROGRESS_HINT  "_NET_WM_XAPP_PROGRESS"
 #define PROGRESS_PULSE_HINT  "_NET_WM_XAPP_PROGRESS_PULSE"
@@ -61,15 +64,25 @@ typedef struct
     gboolean progress_pulse;
 } XAppGtkWindowPrivate;
 
-struct _XAppGtkWindow
-{
-    GtkWindow parent_object;
-
-    XAppGtkWindowPrivate *priv;
-};
-
 G_DEFINE_TYPE_WITH_PRIVATE (XAppGtkWindow, xapp_gtk_window, GTK_TYPE_WINDOW)
 
+static gboolean
+is_x11_session (void)
+{
+    static gboolean running_x11 = FALSE;
+    static gsize once_init_value = 0;
+
+    if (g_once_init_enter (&once_init_value))
+    {
+        running_x11 = GDK_IS_X11_DISPLAY(gdk_display_get_default());
+        DEBUG ("XAppGtkWindow: is_x11_session: %s\n", running_x11 ? "TRUE" : "FALSE");
+
+        g_once_init_leave (&once_init_value, 1);
+    }
+
+    return running_x11;
+}
+
 static void
 clear_icon_strings (XAppGtkWindowPrivate *priv)
 {
@@ -129,7 +142,7 @@ set_window_hint_cardinal (Window       xid,
                          gdk_x11_get_xatom_by_name_for_display (display, atom_name));
     }
 
-    gdk_error_trap_pop ();
+    gdk_error_trap_pop_ignored ();
 }
 
 static Window
@@ -152,6 +165,10 @@ static void
 update_window_icon (GtkWindow            *window,
                     XAppGtkWindowPrivate *priv)
 {
+    if (!is_x11_session ()) {
+        return;
+    }
+
     /* Icon name/path */
     if (priv->icon_name != NULL)
     {
@@ -177,6 +194,10 @@ static void
 update_window_progress (GtkWindow            *window,
                         XAppGtkWindowPrivate *priv)
 {
+    if (!is_x11_session ()) {
+        return;
+    }
+
     /* Progress: 0 - 100 */
     set_window_hint_cardinal (get_window_xid (window),
                               PROGRESS_HINT,
@@ -315,7 +336,7 @@ static void
 xapp_gtk_window_realize (GtkWidget *widget)
 {
     XAppGtkWindow *window = XAPP_GTK_WINDOW (widget);
-    XAppGtkWindowPrivate *priv = window->priv;
+    XAppGtkWindowPrivate *priv = xapp_gtk_window_get_instance_private (window);
 
     GTK_WIDGET_CLASS (xapp_gtk_window_parent_class)->realize (widget);
 
@@ -333,7 +354,7 @@ static void
 xapp_gtk_window_finalize (GObject *object)
 {
     XAppGtkWindow *window = XAPP_GTK_WINDOW (object);
-    XAppGtkWindowPrivate *priv = window->priv;
+    XAppGtkWindowPrivate *priv = xapp_gtk_window_get_instance_private (window);
 
     clear_icon_strings (priv);
 
@@ -344,10 +365,7 @@ static void
 xapp_gtk_window_init (XAppGtkWindow *window)
 {
     XAppGtkWindowPrivate *priv;
-
-    window->priv = G_TYPE_INSTANCE_GET_PRIVATE (window, XAPP_TYPE_GTK_WINDOW, XAppGtkWindowPrivate);
-    
-    priv = window->priv;
+    priv = xapp_gtk_window_get_instance_private (window);
 
     priv->icon_name = NULL;
     priv->icon_path = NULL;
@@ -396,7 +414,9 @@ xapp_gtk_window_set_icon_name (XAppGtkWindow   *window,
 {
     g_return_if_fail (XAPP_IS_GTK_WINDOW (window));
 
-    set_icon_name_internal (GTK_WINDOW (window), window->priv, icon_name);
+    XAppGtkWindowPrivate *priv = xapp_gtk_window_get_instance_private (window);
+
+    set_icon_name_internal (GTK_WINDOW (window), priv, icon_name);
 }
 
 /**
@@ -418,7 +438,9 @@ xapp_gtk_window_set_icon_from_file (XAppGtkWindow   *window,
 {
     g_return_if_fail (XAPP_IS_GTK_WINDOW (window));
 
-    set_icon_from_file_internal (GTK_WINDOW (window), window->priv, file_name, error);
+    XAppGtkWindowPrivate *priv = xapp_gtk_window_get_instance_private (window);
+
+    set_icon_from_file_internal (GTK_WINDOW (window), priv, file_name, error);
 }
 
 /**
@@ -444,7 +466,9 @@ xapp_gtk_window_set_progress (XAppGtkWindow   *window,
 {
     g_return_if_fail (XAPP_IS_GTK_WINDOW (window));
 
-    set_progress_internal (GTK_WINDOW (window), window->priv, progress);
+    XAppGtkWindowPrivate *priv = xapp_gtk_window_get_instance_private (window);
+
+    set_progress_internal (GTK_WINDOW (window), priv, progress);
 }
 
 /**
@@ -467,9 +491,10 @@ xapp_gtk_window_set_progress_pulse (XAppGtkWindow   *window,
                                     gboolean         pulse)
 {
     g_return_if_fail (XAPP_IS_GTK_WINDOW (window));
-    g_return_if_fail (XAPP_IS_GTK_WINDOW (window));
 
-    set_progress_pulse_internal (GTK_WINDOW (window), window->priv, pulse);
+    XAppGtkWindowPrivate *priv = xapp_gtk_window_get_instance_private (window);
+
+    set_progress_pulse_internal (GTK_WINDOW (window), priv, pulse);
 }
 
 
