@@ -41,6 +41,7 @@
 // function, which clears this pointer and unown's the process's bus name.
 static GDBusObjectManagerServer *obj_server = NULL;
 static guint name_owner_id = 0;
+XAppStatusIconState process_icon_state = XAPP_STATUS_ICON_STATE_NO_SUPPORT;
 
 enum
 {
@@ -88,8 +89,6 @@ typedef struct
     GtkWidget *primary_menu;
     GtkWidget *secondary_menu;
 
-    XAppStatusIconState state;
-
     gchar *name;
     gchar *icon_name;
     gchar *tooltip_text;
@@ -276,7 +275,7 @@ primary_menu_unmapped (GtkWidget  *widget,
 
     DEBUG ("Primary menu unmapped");
 
-    if (icon->priv->state == XAPP_STATUS_ICON_STATE_NATIVE)
+    if (process_icon_state == XAPP_STATUS_ICON_STATE_NATIVE)
     {
         xapp_status_icon_interface_set_primary_menu_is_open (icon->priv->interface_skeleton, FALSE);
     }
@@ -293,7 +292,7 @@ secondary_menu_unmapped (GtkWidget  *widget,
 
     DEBUG ("Secondary menu unmapped");
 
-    if (icon->priv->state == XAPP_STATUS_ICON_STATE_NATIVE)
+    if (process_icon_state == XAPP_STATUS_ICON_STATE_NATIVE)
     {
         xapp_status_icon_interface_set_secondary_menu_is_open (icon->priv->interface_skeleton, FALSE);
     }
@@ -338,7 +337,7 @@ popup_menu (XAppStatusIcon *self,
 
     if (button == GDK_BUTTON_PRIMARY)
     {
-        if (self->priv->state == XAPP_STATUS_ICON_STATE_NATIVE)
+        if (process_icon_state == XAPP_STATUS_ICON_STATE_NATIVE)
         {
             xapp_status_icon_interface_set_primary_menu_is_open (self->priv->interface_skeleton, TRUE);
         }
@@ -351,7 +350,7 @@ popup_menu (XAppStatusIcon *self,
     else
     if (button == GDK_BUTTON_SECONDARY)
     {
-        if (self->priv->state == XAPP_STATUS_ICON_STATE_NATIVE)
+        if (process_icon_state == XAPP_STATUS_ICON_STATE_NATIVE)
         {
             xapp_status_icon_interface_set_secondary_menu_is_open (self->priv->interface_skeleton, TRUE);
         }
@@ -781,19 +780,34 @@ on_name_lost (GDBusConnection *connection,
               const gchar     *name,
               gpointer         user_data)
 {
-    XAppStatusIcon *self = XAPP_STATUS_ICON (user_data);
-
     g_warning ("XAppStatusIcon: lost or could not acquire presence on dbus.  Refreshing.");
 
-    self->priv->fail_counter++;
+    GList *instances = g_dbus_object_manager_get_objects (G_DBUS_OBJECT_MANAGER (obj_server));
+    GList *l;
 
-    refresh_icon (self);
+    for (l = instances; l != NULL; l = l->next)
+    {
+        GObject *instance = G_OBJECT (l->data);
+        XAppStatusIcon *icon = XAPP_STATUS_ICON (g_object_get_data (instance, "xapp-status-icon-instance"));
+
+        if (icon == NULL)
+        {
+            g_warning ("on_name_lost: Could not retrieve xapp-status-icon-instance data: %s", name);
+            continue;
+        }
+
+        icon->priv->fail_counter++;
+        refresh_icon (icon);
+    }
+
+    g_list_free_full (instances, g_object_unref);
 }
 
 static void
 sync_skeleton (XAppStatusIcon *self)
 {
     XAppStatusIconPrivate *priv = self->priv;
+    DEBUG ("Syncing icon properties (%s)", priv->name);
 
     priv->fail_counter = 0;
 
@@ -816,15 +830,31 @@ on_name_acquired (GDBusConnection *connection,
                   const gchar     *name,
                   gpointer         user_data)
 {
-    XAppStatusIcon *self = XAPP_STATUS_ICON (user_data);
+    process_icon_state = XAPP_STATUS_ICON_STATE_NATIVE;
 
-    self->priv->state = XAPP_STATUS_ICON_STATE_NATIVE;
+    GList *instances = g_dbus_object_manager_get_objects (G_DBUS_OBJECT_MANAGER (obj_server));
+    GList *l;
 
-    sync_skeleton (self);
+    for (l = instances; l != NULL; l = l->next)
+    {
+        GObject *instance = G_OBJECT (l->data);
+        XAppStatusIcon *icon = XAPP_STATUS_ICON (g_object_get_data (instance, "xapp-status-icon-instance"));
 
-    DEBUG ("Name acquired on dbus, syncing icon properties. State is now: %s",
-             state_to_str (self->priv->state));
-    g_signal_emit (self, signals[STATE_CHANGED], 0, self->priv->state);
+        if (icon == NULL)
+        {
+            g_warning ("on_name_aquired: Could not retrieve xapp-status-icon-instance data: %s", name);
+            continue;
+        }
+
+        sync_skeleton (icon);
+
+        DEBUG ("Name acquired on dbus, state is now: %s",
+               state_to_str (process_icon_state));
+
+        g_signal_emit (icon, signals[STATE_CHANGED], 0, process_icon_state);
+    }
+
+    g_list_free_full (instances, g_object_unref);
 }
 
 typedef struct
@@ -890,6 +920,8 @@ export_icon_interface (XAppStatusIcon *self)
     xapp_object_skeleton_set_status_icon_interface (self->priv->object_skeleton,
                                                     self->priv->interface_skeleton);
 
+    g_object_set_data (G_OBJECT (self->priv->object_skeleton), "xapp-status-icon-instance", self);
+
     g_dbus_object_manager_server_export_uniquely (obj_server,
                                                   G_DBUS_OBJECT_SKELETON (self->priv->object_skeleton));
 
@@ -942,7 +974,7 @@ connect_with_status_applet (XAppStatusIcon *self)
                                                       G_BUS_NAME_OWNER_FLAGS_DO_NOT_QUEUE,
                                                       on_name_acquired,
                                                       on_name_lost,
-                                                      self,
+                                                      NULL,
                                                       NULL);
     }
 
@@ -988,20 +1020,19 @@ on_gtk_status_icon_embedded_changed (GtkStatusIcon *icon,
     g_return_if_fail (GTK_IS_STATUS_ICON (icon));
 
     XAppStatusIcon *self = XAPP_STATUS_ICON (user_data);
-    XAppStatusIconPrivate *priv = self->priv;
 
     if (gtk_status_icon_is_embedded (icon))
     {
-        priv->state = XAPP_STATUS_ICON_STATE_FALLBACK;
+        process_icon_state = XAPP_STATUS_ICON_STATE_FALLBACK;
     }
     else
     {
-        priv->state = XAPP_STATUS_ICON_STATE_NO_SUPPORT;
+        process_icon_state = XAPP_STATUS_ICON_STATE_NO_SUPPORT;
     }
 
     DEBUG ("Fallback icon embedded_changed. State is now %s",
-             state_to_str (priv->state));
-    g_signal_emit (self, signals[STATE_CHANGED], 0, priv->state);
+             state_to_str (process_icon_state));
+    g_signal_emit (self, signals[STATE_CHANGED], 0, process_icon_state);
 }
 
 static void
@@ -1276,7 +1307,6 @@ xapp_status_icon_init (XAppStatusIcon *self)
 
     self->priv->name = g_strdup (g_get_prgname());
 
-    self->priv->state = XAPP_STATUS_ICON_STATE_NO_SUPPORT;
     self->priv->icon_size = FALLBACK_ICON_SIZE;
     self->priv->icon_name = g_strdup (" ");
 
@@ -1300,7 +1330,9 @@ remove_icon_path_from_bus (XAppStatusIcon *self)
 
         DEBUG ("Removing interface at path '%s'", path);
 
+        g_object_set_data (G_OBJECT (self->priv->object_skeleton), "xapp-status-icon-instance", NULL);
         g_dbus_object_manager_server_unexport (obj_server, path);
+
         self->priv->interface_skeleton = NULL;
         self->priv->object_skeleton = NULL;
 
@@ -1805,7 +1837,7 @@ xapp_status_icon_popup_menu (XAppStatusIcon *icon,
 {
     g_return_if_fail (XAPP_IS_STATUS_ICON (icon));
     g_return_if_fail (GTK_IS_MENU (menu) || menu == NULL);
-    g_return_if_fail (icon->priv->state != XAPP_STATUS_ICON_STATE_NO_SUPPORT);
+    g_return_if_fail (process_icon_state != XAPP_STATUS_ICON_STATE_NO_SUPPORT);
 
     popup_menu (icon,
                 menu,
@@ -1970,9 +2002,9 @@ xapp_status_icon_get_state (XAppStatusIcon *icon)
 {
     g_return_val_if_fail (XAPP_IS_STATUS_ICON (icon), XAPP_STATUS_ICON_STATE_NO_SUPPORT);
 
-    DEBUG ("get_state: %s", state_to_str (icon->priv->state));
+    DEBUG ("get_state: %s", state_to_str (process_icon_state));
 
-    return icon->priv->state;
+    return process_icon_state;
 }
 
 /**
