@@ -536,6 +536,17 @@ unref_proxy (gpointer data)
     }
 }
 
+static void
+clean_tempfiles (XAppSnWatcher *watcher)
+{
+    gchar *cmd = g_strdup_printf ("rm -f %s/xapp-tmp-*.png", xapp_get_tmp_dir ());
+
+    // -Wunused-result
+    if (system(cmd)) ;
+
+    g_free (cmd);
+}
+
 static void
 watcher_startup (GApplication *application)
 {
@@ -624,6 +635,8 @@ watcher_shutdown (GApplication *application)
     g_clear_object (&watcher->connection);
     g_clear_object (&watcher->cancellable);
 
+    clean_tempfiles (watcher);
+
     G_APPLICATION_CLASS (xapp_sn_watcher_parent_class)->shutdown (application);
 }
 
@@ -695,6 +708,11 @@ main (int argc, char **argv)
     if (g_settings_get_boolean (xapp_settings, DEBUG_KEY))
     {
         g_setenv ("G_MESSAGES_DEBUG", "all", TRUE);
+
+        gchar *flags = g_settings_get_string (xapp_settings, DEBUG_FLAGS_KEY);
+        g_setenv ("XAPP_DEBUG", flags, TRUE);
+        g_free (flags);
+
     }
 
     whitelist = g_settings_get_strv (xapp_settings,
